function Simple_Example

% generate data points
n = 10; % size of matrices
k = 10; % number of matrices

p = 1; 
f = 2; % specify the condition number of data points
As = cell(k);
for i = 1 : k
    [O, ~] = qr(randn(n));
    D = diag([rand(1, p) + 1, (rand(1, n - p) + 1) * 10^(-f)]);
    As{i} = O * D * O';
end


% compute initial iterate: the Arithmetic-Harmonic mean
Xinitial = computeAH(As);


% set parameters
SolverParams.method = 'LRBFGS'; % specify the method
SolverParams.LengthSY = 4; % specify the memory size of LRBFGS
SolverParams.HasHHR = 1; % locking condition is satisfied
SolverParams.DEBUG = 3; % output information
SolverParams.LineSearch_LS = 0; % Armijo linesearch
SolverParams.LS_ratio1 = 0.1; % coefficient in the Armijo condition
SolverParams.LS_ratio2 = 0.9; % coefficient in the Armijo condition
SolverParams.InitSteptype = 0; % initial stepsize = 1
SolverParams.Initstepsize = 1.0; % initial stepsize in the first iteration
SolverParams.Max_Iteration = 50; % maximum number of iterations
SolverParams.Stop_Criterion = 2; % stopping criterion
SolverParams.Tolerance = 1e-6; % algorithm stops if Stop_Criterion < tolerance
SolverParams.Accuracy = 1e-5; % skip the linesearch procedure if |gf|/|gf0| < accuracy
SolverParams.IsCheckGradHess = 0; % check the correctness of gradient and Hessian

% for more information regarding the parameters, see Section B in 
% http://www.math.fsu.edu/~whuang2/pdf/USER_MANUAL_for_2016-10-26.pdf


% Xtrue is needed if one wants to compute the true error at each iteration
% Otherwise make it an identity matrix or any other SPD matrix
Xtrue = eye(n); 

% compute the Karcher mean
[SPDMean, f, gf, gfgf0, iter, nf, ng, nR, nV, nVp, nH, ComTime, ...
    funs, grads, times, diss, initstepsizes, stepsizes, truestepsizes, relfs] = ... 
    ComputeSPDKarcherMean(As, Xinitial, Xtrue, SolverParams);

% plot the norm of gradient at each iteration
figure(1)
h = semilogy(1,100);
set(get(get(h,'Annotation'),'LegendInformation'),'IconDisplayStyle','off');
hold on;
set(gca,'FontSize',13)
plot(grads,'-square','markersize',12,'linewidth',2);
xlabel('iterations','FontSize',16,'Interpreter','Latex');
ylabel('|grad|','FontSize',16,'Interpreter','Latex');


% plot the function value at each iteration
figure(2)
set(gca,'FontSize',13)
plot(funs,'-square','markersize',12,'linewidth',2);
xlabel('iterations','FontSize',16,'Interpreter','Latex');
ylabel('f','FontSize',16,'Interpreter','Latex');
end